
__version__ = "0.5.0"

PPM =20

from pygame_phyics.manger import Manger

from pygame_phyics.game import Game
import pygame_phyics.game as game


import pygame_phyics.mouse as mouse
import pygame_phyics.object as object

from pygame_phyics.object import GameObject

from pygame_phyics.object import StaticObject
from pygame_phyics.object import DynamicObject
from pygame_phyics.object import ImageObject

from pygame_phyics.sheet import SurfaceSheet

